package test.rewolucja.resources.physical.implementation;

import java.util.Properties;

import schedframe.resources.units.CpuSpeed;
import schedframe.scheduling.utils.ResourceParameterName;
import test.rewolucja.energy.extension.EnergyExtension;
import test.rewolucja.energy.profile.PowerInterface;
import test.rewolucja.energy.profile.implementation.CPUPowerProfile;
import test.rewolucja.extensions.ExtensionType;
import test.rewolucja.resources.ResourceCharacteristics;
import test.rewolucja.resources.description.ExecResourceDescription;
import test.rewolucja.resources.physical.base.ComputingResource;
import test.rewolucja.resources.properties.CpuPropertiesBuilder;
import test.rewolucja.resources.properties.PropertiesDirector;

public class Processor extends ComputingResource{
	

	public Processor (ExecResourceDescription resDesc) {
		super(resDesc);
		accept(resDesc.getPowerProfile());
	}
	
	/*public CPU (String resourceName, ResourceCharacteristics resourceCharacteristic) {
		super(resourceName, ResourceType.CPU, resourceCharacteristic);
		extensionList.add(new EnergyExtension(this, "example.energy.CPUEnergyEstimationPlugin"));
	}*/

	public ComputingNode getComputingNode(){
		return (ComputingNode)parent;
	}
	
	public int getMIPS(){
		return getSpeedUnit().getAmount();
	}
	
	public int getAvailableMIPS(){
		return getSpeedUnit().getFreeAmount();
	}

	private CpuSpeed getSpeedUnit(){
		return (CpuSpeed) getResourceCharacteristic().getResourceUnit(ResourceParameterName.CPUSPEED);
	}
	
	public CPUPowerProfile getPowerInterface(){
		if (extensionList != null) {
			if(extensionList.isExtensionAvailable(ExtensionType.ENERGY_EXTENSION)){
				EnergyExtension ee = (EnergyExtension)extensionList.getExtension(ExtensionType.ENERGY_EXTENSION);
				return (CPUPowerProfile)ee.getPowerInterface();
			}
		}
		return null;
	}
	
	public void initCharacteristics(ExecResourceDescription resDesc){
		resourceCharacteristic = new ResourceCharacteristics();
		
		int speed = 1000;
		if( resDesc.getCategory().getName().equals("A"))
			speed = 2000;
		
		resourceCharacteristic.addResourceUnit(new CpuSpeed(name, speed, 0));
	}
	
	public Properties getProperties(){
		PropertiesDirector propDirector = new PropertiesDirector();
		propDirector.setPropertiesBuilder(new CpuPropertiesBuilder());
		propDirector.constructProperties(this);
		return propDirector.getProperties();
	}

	public void accept(PowerInterface powIntVis){
		extensionList.add(new EnergyExtension(this, powIntVis, "example.energy.CPUEnergyEstimationPlugin"));
		powIntVis.visit(this);
	}
}

